import os
import subprocess
import sys
from src.util import wait_for_elastic_search

import click
from termcolor import colored

import libtmux


@click.command(help="Proxy for yarn kbn bootstrap")
@click.option("--clean", "-c", is_flag=True, help="Runs bootstrap with clean")
@click.option(
    "--tmuxrun",
    is_flag=True,
    help=(
        "If you run kibbe inside a tmux session it will divide the current window, run"
        " elasticsearch (kibbe es) and kibana (kibbe kibana)"
    ),
)
def setmeup(clean, tmuxrun):

    commands = []
    if clean:
        commands.append(["yarn", "kbn", "bootstrap", "clean"])

    commands.append(["yarn", "kbn", "bootstrap"])

    for command in commands:
        subprocess.run(command)

    if tmuxrun:
        setup_tmux()


def setup_tmux():

    current_window = get_current_window()
    clean_tmux_window(current_window)
    current_pane_id = os.getenv("TMUX_PANE")
    current_pane = current_window.get_by_id(current_pane_id)

    es_pane = current_window.split_window(vertical=False)
    start_and_wait_for_es(es_pane)

    click.echo(colored("Starting kibana and closing this pane", "yellow"))
    kibana_pane = current_window.split_window(vertical=False)
    start_kibana(kibana_pane)

    # close this pane. This will also kill the python process
    current_pane.cmd("kill-pane")
    exit()


def start_kibana(kibana_pane):
    current_dir = os.getcwd()
    kibbe_command = sys.argv[0]

    kibana_pane.send_keys("cd %s" % current_dir)
    kibana_pane.send_keys("nvm use")
    kibana_pane.send_keys("%s kibana" % kibbe_command)


def start_and_wait_for_es(es_pane):
    current_dir = os.getcwd()
    kibbe_command = sys.argv[0]

    es_pane.send_keys("cd %s" % current_dir)
    es_pane.send_keys("nvm use")
    es_pane.send_keys("%s es" % kibbe_command)

    click.echo(
        colored(
            "echo Waiting for elasticsearch to start... If you run elasticsearch in a"
            " non-standard port this will timeout after 60 seconds",
            "blue",
        )
    )

    timeout = wait_for_elastic_search()
    if timeout:
        click.echo(
            colored("elasticsearch timeout. Continuing with kibana anyway", "red")
        )
    else:
        click.echo(colored("elasticsearch is ready", "blue"))


def get_current_window():
    tmux_panel = os.getenv("TMUX_PANE")
    tmux_session = os.getenv("TMUX")
    if not tmux_panel or not tmux_session:
        raise click.ClickException("Not inside a tmux session")

    tmux_session_id = "$" + tmux_session.split(",")[-1]
    server = libtmux.Server()

    session = server.get_by_id(tmux_session_id)
    if not session:
        raise click.ClickException("Could not interact with Tmux. Wrong tmux version")

    return session.attached_window


def clean_tmux_window(current_window):

    if len(current_window.panes) > 1:
        if not click.confirm(
            colored(
                "The current window has more than 1 pane. Do you want to close them all"
                " and start new ones?",
                "yellow",
            )
        ):
            raise click.ClickException(
                colored(
                    "Close all panes or create a new window to continue and run kibbe"
                    " again",
                    "red",
                )
            )
        current_pane_id = os.getenv("TMUX_PANE")
        current_pane = current_window.get_by_id(current_pane_id)
        panes = iter(current_window.panes)
        for pane in panes:
            if pane != current_pane:
                pane.cmd("kill-pane")



import click

from src.commands.check import check
from src.commands.es import es
from src.commands.kibana import kibana
from src.commands.setlicense import setlicense
from src.commands.setmeup import setmeup
from src.config import set_config_file
from src.util import force_kibana_root


@click.group(
    help="""
        Kibbe is a tool that help with common tasks when developing kibana plugins.

        Some subcommands allow you to define a configuration to persist arguments for some
        specific tasks such as running kibana or elasticsearch. This configuration file will
        persis those arguments among all kibana clones and branches.

        You can create a configuration file in your home ~/.kibbe and follow the configuration
        example in the kibbe repository
"""
)
@click.option(
    "--config-file", help="Overwrites the default (~/.kibbe) config file location"
)
def cli(config_file):
    set_config_file(config_file)
    force_kibana_root()


cli.add_command(setlicense)
cli.add_command(check)
cli.add_command(es)
cli.add_command(kibana)
cli.add_command(setmeup)



from pathlib import Path
import re
import subprocess
import tempfile

import click
from termcolor import colored

from src.config import get_config, persist_config
from src.util import merge_params, unparsed_to_map

pathDataRe = re.compile(r"path\.data\s?=", re.IGNORECASE)


@click.command(
    context_settings=dict(
        ignore_unknown_options=True,
    )
)
@click.option(
    "--data-dir",
    "-d",
    type=click.STRING,
    default="",
    help="Path where this elastic search will store its data (path.data)",
)
@click.option(
    "--no-persist",
    "-n",
    default=False,
    is_flag=True,
    help=(
        "If passed will use a disposable data dir. This option will overwrite other"
        " options related to data dir."
    ),
)
@click.option(
    "--save-config",
    default=False,
    is_flag=True,
    help=(
        "If passed it will write your kibbe configuration with all the current passed"
        " parameters. This will not modify your kibana repo clone."
    ),
)
@click.option(
    "-E",
    multiple=True,
    help="Additional options to pass to elastic search. `path.data` will be ignored",
)
@click.argument("unparsed_args", nargs=-1, type=click.UNPROCESSED)
def es(data_dir, no_persist, e, unparsed_args, save_config):
    """
    Runs elastic search from the current kibana clone.

    You can also pass the same parameters as you'd pass to `node scritps/es`

    You can persist the -E parameters by using a configuration file `~/.kibbe`.
    with the [elastic.params] section.

    See more about the configuration file here:
    https://github.com/academo/kibbe#configuration-file
    """

    e_params = get_params(data_dir, no_persist)

    # additional -E params
    for item in e:
        item = item.strip()
        # ignore path.data
        if pathDataRe.match(item):
            continue
        e_params.append(item)

    params = []
    config = get_config()

    config_params = []
    if "elastic.params" in config:
        config_params = config.items("elastic.params", raw=True)
    params = merge_params(config_params, unparsed_args)

    if save_config:
        persist_config(
            {"elastic.eparams": e_params, "elastic.params": unparsed_to_map(params)}
        )
        exit()

    command = get_command(e_params, extra_params=params)
    click.echo("Will run elastic search as: " + colored(" ".join(command), "yellow"))
    subprocess.run(command)


def get_command(e_params, extra_params):
    final_params = []
    for param in e_params:
        final_params.append("-E")
        final_params.append(param)

    return ["node", "scripts/es", "snapshot"] + final_params + extra_params


def get_params(data_dir, no_persist):
    CONFIG_KEY = "elastic.eparams"
    config = get_config()
    params = []
    if CONFIG_KEY in config:
        for (key, value) in config.items(CONFIG_KEY, raw=True):
            # ignore path.data if this command overwrites it
            if key == "path.data":
                if len(data_dir) > 0:
                    value = get_data_dir(data_dir, no_persist)
                else:
                    value = get_data_dir(value, no_persist)

            if len(value) > 0:
                params.append(str(key) + "=" + str(value))
            else:
                params.append(str(key))

    return params


def get_data_dir(data_dir, no_persist):
    if no_persist or len(data_dir) == 0:
        return tempfile.mkdtemp(suffix="kibbe")

    return str(Path(data_dir).resolve())



import subprocess
import click
from src.config import get_config, persist_config
from src.util import merge_params, unparsed_to_map
from src.util import wait_for_elastic_search
from termcolor import colored


@click.command(
    context_settings=dict(
        ignore_unknown_options=True,
    )
)
@click.option(
    "--wait",
    "-w",
    default=False,
    is_flag=True,
    help=(
        "If passed. It will wait for an elastic search instance in the default port"
        " (9200) to be ready before starting kibana"
    ),
)
@click.option(
    "--save-config",
    default=False,
    is_flag=True,
    help=(
        "If passed it will write your kibbe configuration with all the current passed"
        " parameters. This will not modify your kibana repo clone."
    ),
)
@click.argument("unparsed_args", nargs=-1, type=click.UNPROCESSED)
def kibana(save_config, unparsed_args, wait):
    """
    Runs Kibana from the current clone.

    You can pass the same parameters as you'd pass to `node scritps/kibana`

    You can persist some parameters by using a configuration file `~/.kibbe`.
    with the [kibana.params] section.

    See more about the configuration file here:
    https://github.com/academo/kibbe#configuration-file
    """

    if wait:
        click.echo(
            colored("Waiting for elasticsearch in port 9200. Timeout in 60s", "blue")
        )
        wait_for_elastic_search()

    config_params = []
    config = get_config()
    if "kibana.params" in config:
        config_params = config.items("kibana.params", raw=True)

    params = merge_params(config_params, unparsed_args)

    if save_config:
        persist_config({"kibana.params": unparsed_to_map(params)})
        exit()

    command = ["node", "scripts/kibana", "--dev"] + params
    click.echo("Will run kibana search as: " + colored(" ".join(command), "yellow"))
    subprocess.run(command)



import configparser
from pathlib import Path, PurePath

import click
from termcolor import colored

config_path = ""


def set_config_file(path):
    global config_path
    if path and len(path) > 0:
        config_path = str(PurePath(path))
    else:
        config_path = str(PurePath(Path.home()).joinpath(".kibbe"))


def get_config_file():
    global config_path
    return config_path


def get_config():
    config = configparser.ConfigParser()
    try:
        config.read(get_config_file())
    except ValueError:
        click.echo(colored("Error reading your configuration file.", "red"))
        pass
    return config


def persist_config(config_map):

    config = get_config_to_save(config_map)

    try:
        print_config(config)
        if click.confirm(
            "Are you sure you want to save this configuration?\nAll existing"
            " configuration will be overwritten"
        ):
            config_file = open(get_config_file(), "w")
            config.write(config_file)
            click.echo(colored("Configuration written to kibbe config", "blue"))
        else:
            click.echo(colored("Cancelled", "yellow"))
            exit()
    except ValueError:
        exit()


def get_config_to_save(config_map):
    config = get_config()
    for config_key in config_map:
        if config_key not in config:
            config.add_section(config_key)

        data = config_map[config_key]
        if type(data) is dict:
            for key in data:
                wkey = key
                if wkey.startswith("--"):
                    wkey = wkey[2:]
                config.set(config_key, wkey, data[key])
        else:
            for value in data:
                value = value.split("=")
                if len(value) > 1:
                    config.set(config_key, value[0], value[1])
                else:
                    config.set(config_key, value[0], "")

    return config


def print_config(config):
    config_string = "--start--\n"
    sections = config.sections()
    for section in sections:
        config_string = config_string + "[%s]\n" % (section)
        items = config.items(section)
        for item, value in items:
            config_string = config_string + "%s = %s\n" % (item, value)
        config_string = config_string + "\n"
    config_string = config_string + "---end---"

    click.echo(colored("Config to save:\n ", "yellow"))
    click.echo(config_string)
